@@ -1,13 +1,11 @@
-# $Id: /mirror/coderepos/lang/perl/Data-Localize/trunk/lib/Data/Localize.pm 103047 2009-04-01T01:39:18.931794Z daisuke  $
-
 package Data::Localize;
 use Any::Moose;
 use Any::Moose '::Util::TypeConstraints';
-use Any::Moose 'X::AttributeHelpers';
 use I18N::LangTags ();
 use I18N::LangTags::Detect ();
+use 5.008;
 
-our $VERSION = '0.00006_01';
+our $VERSION = '0.00013_01';
 our $AUTHORITY = 'cpan:DMAKI';
 
 BEGIN {
@@ -20,38 +18,30 @@ BEGIN {
     }
 }
 
-has 'auto' => (
+has auto => (
     is => 'rw',
     isa => 'Bool',
     default => 1,
 );
 
-has 'auto_localizer' => (
-    is => 'rw',
+has auto_localizer => (
+    is => 'ro',
     isa => 'Data::Localize::Auto',
     lazy_build => 1,
 );
 
-has 'languages' => (
-    metaclass => 'Collection::Array',
+has _languages => (
     is => 'rw',
     isa => 'ArrayRef',
-    auto_deref => 1,
     lazy_build => 1,
-    provides => {
-        push => 'add_languages',
-    }
+    init_arg => 'languages',
 );
 
-has 'fallback_languages' => (
-    metaclass => 'Collection::Array',
+has _fallback_languages => (
     is => 'rw',
     isa => 'ArrayRef',
-    auto_deref => 1,
     lazy_build => 1,
-    provides => {
-        push => 'add_fallback_languages',
-    }
+    init_arg => 'languages',
 );
 
 # Localizers are the actual minions that perform the localization.
@@ -81,28 +71,18 @@ coerce 'Data::Localize::LocalizerListArg'
     }
 ;
 
-has 'localizers' => (
-    metaclass => 'Collection::Array',
+has _localizers => (
     is => 'rw',
     isa => 'Data::Localize::LocalizerListArg',
     coerce => 1,
     default => sub { +[] },
-    provides => {
-        'push'  => 'push_localizers',
-        'count' => 'count_localizers',
-        'grep'  => 'grep_localizers',
-    }
+    init_arg => 'localizers',
 );
 
-has 'localizer_map' => (
-    metaclass => 'Collection::Hash',
-    is => 'rw',
+has localizer_map => (
+    is => 'ro',
     isa => 'HashRef',
     default => sub { +{} },
-    provides => {
-        get => 'get_localizer_from_lang',
-        set => 'set_localizer_map',
-    }
 );
 
 __PACKAGE__->meta->make_immutable;
@@ -113,25 +93,72 @@ no Any::Moose '::Util::TypeConstraints';
 sub BUILD {
     my $self = shift;
     if ($self->count_localizers > 0) {
-        foreach my $loc (@{ $self->localizers }) {
+        foreach my $loc (@{ $self->_localizers }) {
             $loc->register($self);
         }
     }
     return $self;
 }
 
-sub _build_fallback_languages {
-    return [];
+sub _build__fallback_languages {
+    return [ 'en' ];
 }
 
-sub _build_languages {
+sub _build__languages {
     my $self = shift;
     $self->detect_languages();
 }
 
 sub _build_auto_localizer {
+    my $self = shift;
     require Data::Localize::Auto;
-    Data::Localize::Auto->new;
+    Data::Localize::Auto->new();
+}
+
+sub set_languages {
+    my $self = shift;
+    $self->_languages([ @_ > 0 ? @_ : $self->detect_languages ]);
+};
+
+
+sub add_fallback_languages {
+    my $self = shift;
+    push @{$self->_fallback_languages}, @_;
+}
+
+sub fallback_languages {
+    my $self = shift;
+    return @{$self->_fallback_languages};
+}
+
+sub languages {
+    my $self = shift;
+    return @{$self->_languages};
+}
+
+sub localizers {
+    my $self = shift;
+    return $self->_localizers;
+}
+
+sub count_localizers {
+    my $self = shift;
+    return scalar @{$self->_localizers};
+}
+
+sub grep_localizers {
+    my ($self, $cb) = @_;
+    grep { $cb->($_) } @{$self->_localizers};
+}
+
+sub get_localizer_from_lang {
+    my ($self, $key) = @_;
+    return $self->localizer_map->{$key};
+}
+
+sub set_localizer_map {
+    my ($self, $key, $value) = @_;
+    return $self->localizer_map->{$key} = $value;
 }
 
 sub detect_languages {
@@ -140,7 +167,7 @@ sub detect_languages {
         I18N::LangTags::Detect::detect() ||
         $self->fallback_languages,
     );
-    if (&DEBUG) {
+    if (DEBUG()) {
         print STDERR "[Data::Localize]: detect_languages auto-detected ", join(", ", map { "'$_'" } @lang ), "\n";
     }
     return wantarray ? @lang : \@lang;
@@ -152,17 +179,12 @@ sub detect_languages_from_header {
         I18N::LangTags::Detect->http_accept_langs( $_[0] || $ENV{HTTP_ACCEPT_LANGUAGE}),
         $self->fallback_languages,
     );
-    if (&DEBUG) {
+    if (DEBUG()) {
         print STDERR "[Data::Localize]: detect_languages_from_header detected ", join(", ", map { "'$_'" } @lang ), "\n";
     }
     return wantarray ? @lang : \@lang;
 }
 
-sub set_languages {
-    my $self = shift;
-    $self->languages([ @_ > 0 ? @_ : $self->detect_languages ]);
-}
-
 sub localize {
     my ($self, $key, @args) = @_;
 
@@ -184,7 +206,7 @@ sub localize {
     # one last shot. try the '*' slot
     foreach my $localizer (@{$self->get_localizer_from_lang('*') || []}) {
         foreach my $lang ($self->languages) {
-            if (DEBUG) {
+            if (DEBUG()) {
                 print STDERR "[Data::Localize]: localize - trying $lang for '*' with localizer $localizer\n" if DEBUG;
             }
             my $out = $localizer->localize_for(
@@ -205,7 +227,7 @@ sub localize {
     # if you got here, and you /still/ can't find a proper localization,
     # then we fallback to 'auto' feature
     if ($self->auto) {
-        if (DEBUG) {
+        if (DEBUG()) {
             print STDERR "[Data::Localize]: localize - trying auto-lexicon for $key\n";
         }
         return $self->auto_localizer->localize_for(id => $key, args => \@args);
@@ -232,7 +254,7 @@ sub add_localizer {
     }
 
     $localizer->register($self);
-    $self->push_localizers($localizer);
+    push @{ $self->_localizers }, $localizer;
 }
 
 sub find_localizers {
@@ -246,7 +268,7 @@ sub find_localizers {
 sub add_localizer_map {
     my ($self, $lang, $localizer) = @_;
 
-    if (DEBUG) {
+    if (DEBUG()) {
         print STDERR "[Data::Localize]: add_localizer_map $lang -> $localizer\n";
     }
     my $list = $self->get_localizer_from_lang($lang);
@@ -298,6 +320,27 @@ Data::Localize is an object oriented approach to localization, aimed to
 be an alternate choice for Locale::Maketext, Locale::Maketext::Lexicon, and
 Locale::Maketext::Simple.
 
+=head1 RATIONALE
+
+Functionality-wise, Locale::Maketext does what it advertises to do.
+Here's a few reasons why you might or might not choose Data::Localize
+over Locale::Maketext-based localizers:
+
+=head2 Object-Oriented
+
+Data::Localize is completely object-oriented. YMMV.
+
+=head2 Faster
+
+On my benchmarks, Data::Localize is consistently faster than Locale::Maketext
+by 50~80%
+
+=head2 Scalable For Large Amount Of Lexicons
+
+Whereas Locale::Maketext generally stores the lexicons in memory,
+Data::Localize allows you to store this data in alternate storage.
+By default Data::Localize comes with a BerkeleyDB backend.
+
 =head1 BASIC WORKING 
 
 =head2 STRUCTURE
@@ -322,20 +365,48 @@ Locale::Maketext allows you to supply an "_AUTO" key in the lexicon hash,
 which allows you to pass a non-existing key to the localize() method, and
 use it as the actual lexicon, if no other applicable lexicons exists.
 
-    # here, we're deliberately not setting any localizers
-    my $loc = Data::Localize->new(auto => 1);
-
-    print $loc->localize('Hello, [_1]', 'John Doe'), "\n";
-
 Locale::Maketext attaches this to the lexicon hash itself, but Data::Localizer
 differs in that it attaches to the Data::Localizer object itself, so you
 don't have to place _AUTO everwhere.
 
+    # here, we're deliberately not setting any localizers
+    my $loc = Data::Localize->new(auto => 1);
+
+    # previous auto => 1 will force Data::Localize to fallback to
+    # using the key ('Hello, [_1]') as the localization token.
+    print $loc->localize('Hello, [_1]', 'John Doe'), "\n";
+
 =head1 UTF8
 
-All data is expected to be in decoded utf8. You must "use utf8" for all values
+All data is expected to be in decoded utf8. You must "use utf8" or 
+decode them to Perl's internal representation for all values
 passed to Data::Localizer. We won't try to be smart for you. USE UTF8!
 
+=over 4
+
+=item Using Explicit decode()
+
+    use Encode q(decode decode_utf8);
+    use Data::Localizer;
+
+    my $loc = Data::Localize->new(...);
+
+    $loc->localize( $key, decode( 'iso-2022-jp', $value ) );
+
+    # if $value is encoded utf8...
+    # $loc->localize( $key, decode_utf8( $value ) );
+
+=item Using utf8
+
+"use utf8" is simpler, but do note that it will affect ALL your literal strings
+in the current scope
+
+    use utf8;
+
+    $loc->loclize( $key, "some-utf8-key-here" );
+
+=back
+
 =head1 USING ALTERNATE STORAGE
 
 By default all lexicons are stored on memory, but if you're building an app
@@ -343,7 +414,7 @@ with thousands and thousands of long messages, this might not be the ideal
 solution. In such cases, you can change where the lexicons get stored
 
     my $loc = Data::Localize->new();
-    $loc->add_namespace(
+    $loc->add_localizer(
         class         => 'Gettext',
         path          => '/path/to/data/*.po'
         storage_class => 'BerkeleyDB',
@@ -401,10 +472,18 @@ I18N::LanguageTags::Detect for details.
 
 Detects the language from the given header value, or from HTTP_ACCEPT_LANGUAGES environment variable
 
+=head2 localizers
+
+Return a arrayref of localizers
+
 =head2 add_localizer_map
 
 Used internally.
 
+=head2 set_localizer_map
+
+Used internally.
+
 =head2 find_localizers 
 
 Finds a localizer by its attribute. Currently only supports isa
@@ -416,11 +495,37 @@ Finds a localizer by its attribute. Currently only supports isa
 If used without any arguments, calls detect_languages() and sets the
 current language set to the result of detect_languages().
 
+=head2 languages
+
+Getht the current list of languages
+
+=head2 add_fallback_languages
+
+=head2 fallback_languages
+
+XXX - Consider if fallback_languages can be removed
+
+=head2 count_localizers()
+
+Return the number of localizers available
+
+=head2 get_localizer_from_lang($lang)
+
+Get appropriate localizer for language $lang
+
+=head2 grep_localizers(\&sub)
+
+Filter localizers
+
 =head1 TODO
 
 Gettext style localization files -- Make it possible to decode them
 Check performance -- see benchmark/benchmark.pl
 
+=head1 CONTRIBUTORS
+
+Dave Rolsky
+
 =head1 AUTHOR
 
 Daisuke Maki C<< <daisuke@endeworks.jp> >>
