@@ -1,24 +1,21 @@
-# $Id: /mirror/coderepos/lang/perl/Data-Localize/trunk/lib/Data/Localize/Auto.pm 103047 2009-04-01T01:39:18.931794Z daisuke  $
 
 package Data::Localize::Auto;
 use Any::Moose;
 
-with 'Data::Localize::Localizer';
+extends 'Data::Localize::Localizer';
 
 __PACKAGE__->meta->make_immutable;
 
 no Any::Moose;
 
-sub register {}
-
-sub lexicon_get {}
+sub get_lexicon {}
 
 sub localize_for {
     my ($self, %args) = @_;
     my ($id, $args) = @args{ qw(id args) };
 
     my $value = $id;
-    if (&Data::Localize::DEBUG) {
+    if (Data::Localize::DEBUG()) {
         print STDERR "[Data::Localize::Auto]: localize_for - $id -> ",
             defined($value) ? $value : '(null)', "\n";
     }
@@ -40,7 +37,7 @@ Data::Localize::Auto - Fallback Localizer
 
 =head2 register
 
-=head2 lexicon_get
+=head2 get_lexicon
 
 Does nothing
 
