@@ -1,29 +1,35 @@
-# $Id: /mirror/coderepos/lang/perl/Data-Localize/trunk/lib/Data/Localize/Namespace.pm 103044 2009-04-01T01:17:04.738082Z daisuke  $
 
 package Data::Localize::Namespace;
 use Any::Moose;
-use Any::Moose 'X::AttributeHelpers';
 use Module::Pluggable::Object;
 use Encode ();
+use Data::Localize::Util qw(_alias_and_deprecate);
 
-with 'Data::Localize::Localizer';
+extends 'Data::Localize::Localizer';
 
-has 'namespaces' => (
-    metaclass => 'Collection::Array',
+has _namespaces => (
     is => 'rw',
     isa => 'ArrayRef',
-    auto_deref => 1,
-    provides => {
-        unshift => 'add_namespaces'
-    }
+    default => sub { [] },
+    init_arg => 'namespaces',
 );
 
-__PACKAGE__->meta->make_immutable;
+has _loaded_classes => (
+    is => 'ro',
+    isa => 'HashRef',
+    default => sub { +{} }
+);
 
-no Any::Moose;
+has _failed_classes => (
+    is => 'ro',
+    isa => 'HashRef',
+    default => sub { +{} }
+);
 
-sub register {
+override register => sub {
     my ($self, $loc) = @_;
+    super();
+
     my $finder = Module::Pluggable::Object->new(
         'require' => 1,
         search_path => [ $self->namespaces ]
@@ -37,49 +43,81 @@ sub register {
         $loc->add_localizer_map($plugin, $self);
     }   
     $loc->add_localizer_map('*', $self);
+};
+
+_alias_and_deprecate lexicon_get => 'get_lexicon';
+
+__PACKAGE__->meta->make_immutable;
+
+no Any::Moose;
+
+sub add_namespaces {
+    my $self = shift;
+    unshift @{ $self->_namespaces }, @_;
 }
 
-our %LOADED;
-sub lexicon_get {
+sub namespaces {
+    my $self = shift;
+    return @{ $self->_namespaces };
+}
+
+sub get_lexicon {
     my ($self, $lang, $id) = @_;
 
+    $lang =~ s/-/_/g;
+
+    my $LOADED = $self->_loaded_classes;
+    my $FAILED = $self->_failed_classes;
     foreach my $namespace ($self->namespaces) {
         my $klass = "$namespace\::$lang";
-        if (&Data::Localize::DEBUG) {
-            print STDERR "[Data::Localize::Namespace]: lexicon_get - Trying $klass\n";
+
+        if ($FAILED->{ $klass }++) {
+            if (Data::Localize::DEBUG()) {
+                print STDERR "[Data::Localize::Namespace]: get_lexicon - Already attempted loading $klass and failed. Skipping...\n";
+            }
+            next;
+        }
+
+        if (Data::Localize::DEBUG()) {
+            print STDERR "[Data::Localize::Namespace]: get_lexicon - Trying $klass\n";
         }
 
         # Catch the very weird case where is_class_loaded() returns true
         # but the class really hasn't been loaded yet.
         no strict 'refs';
         my $first_load = 0;
-        if (! $LOADED{$klass}) {
+        if (! $LOADED->{$klass}) {
             if (defined %{"$klass\::Lexicon"} && defined %{"$klass\::"}) {
-                if (&Data::Localize::DEBUG) {
-                    print STDERR "[Data::Localize::Namespace]: lexicon_get - class already loaded\n";
+                if (Data::Localize::DEBUG()) {
+                    print STDERR "[Data::Localize::Namespace]: get_lexicon - class already loaded\n";
                 }
             } else {
-                if (&Data::Localize::DEBUG) {
-                    print STDERR "[Data::Localize::Namespace]: lexicon_get - loading $klass\n";
+                if (Data::Localize::DEBUG()) {
+                    print STDERR "[Data::Localize::Namespace]: get_lexicon - loading $klass\n";
                 }
 
-                my $code = "\n#line " . __LINE__ . ' "' . __FILE__ . "\"\nrequire $klass";
-                eval $code;
+                my $code = 
+                    "\n" .
+                    "#line " . __LINE__ . ' "' . __FILE__ . '"' . "\n" .
+                    "require $klass;"
+                ;
+                eval($code);
                 if ($@) {
-                    if (&Data::Localize::DEBUG) {
-                        print STDERR "[Data::Localize::Namespace]: lexicon_get - Failed to load $klass: $@\n";
+                    if (Data::Localize::DEBUG()) {
+                        print STDERR "[Data::Localize::Namespace]: get_lexicon - Failed to load $klass: $@\n";
+                        $FAILED->{$klass}++;
                     }
                     next;
                 }
             }
-            if (&Data::Localize::DEBUG) {
-                print STDERR "[Data::Localize::Namespace]: lexicon_get - setting $klass to already loaded\n";
+            if (Data::Localize::DEBUG()) {
+                print STDERR "[Data::Localize::Namespace]: get_lexicon - setting $klass to already loaded\n";
             }
-            $LOADED{$klass}++;
+            $LOADED->{$klass}++;
             $first_load = 1;
         }
 
-        if (&Data::Localize::DEBUG) {
+        if (Data::Localize::DEBUG()) {
             print STDERR "[Data::Localize::Namespace]: returning lexicon from $klass (", scalar keys %{"$klass\::Lexicon"}, " lexicons)\n";
         }
         my $h = \%{ "$klass\::Lexicon" };
@@ -116,7 +154,7 @@ Data::Localize::Namespace - Acquire Lexicons From Module %Lexicon Hash
    package MyApp::I18N::ja;
    use strict;
    our %Lexicon = (
-      "Hello, %1!" => "%1さん、こんにちは!"
+      "Greeting" => "[_1]さん、こんにちは!"
    );
 
    1;
@@ -124,26 +162,34 @@ Data::Localize::Namespace - Acquire Lexicons From Module %Lexicon Hash
    use Data::Localize;
 
    my $loc = Data::Localize::Namespace->new(
-      style => "gettext",
       namespace => "MyApp::I18N",
    );
    my $out = $loc->localize_for(
       lang => 'ja',
-      id   => 'Hello, %1!',
+      id   => 'Greeting',
       args => [ 'John Doe' ]
    );
 
 =head1 METHODS
 
-=head2 register
+=head2 add_namespaces
 
-Registeres this localizer to the Data::Localize object
+Add a new namespace to the END of the namespace list
 
-=head2 lexicon_get 
+=head2 get_lexicon 
 
 Looks up lexicon data from given namespaces. Packages must be discoverable
 via Module::Pluggable::Object, with a package name like YourNamespace::lang
 
+=head2 namespaces
+
+Get all the namespaces that this localizer will look up, in the order that
+they will be looked up.
+
+=head2 register
+
+Registeres this localizer to the Data::Localize object
+
 =head1 AUTHOR
 
 Daisuke Maki C<< <daisuke@endeworks.jp> >>
